@@ -7,8 +7,13 @@ use vars qw($VERSION);
 use base qw/DBIx::Class::Componentised Class::Data::Accessor/;
 
 sub mk_classdata { shift->mk_classaccessor(@_); }
+sub component_base_class { 'DBIx::Class' }
 
-$VERSION = '0.04999_01';
+# Always remember to do all digits for the version even if they're 0
+# i.e. first release of 0.XX *must* be 0.XX000. This avoids fBSD ports
+# brain damage and presumably various other packaging systems too
+
+$VERSION = '0.04999_02';
 
 1;
 
@@ -118,6 +123,14 @@ Marcus Ramberg <mramberg@cpan.org>
 
 Paul Makepeace
 
+CL Kao
+
+Jess Robinson
+
+Marcus Ramberg
+
+Will Hawes
+
 =head1 LICENSE
 
 You may distribute this code under the same terms as Perl itself.
