@@ -15,7 +15,8 @@ sub inject_base {
 
 sub load_components {
   my $class = shift;
-  my @comp = map { "DBIx::Class::$_" } grep { $_ !~ /^#/ } @_;
+  my $base = $class->component_base_class;
+  my @comp = map { "${base}::$_" } grep { $_ !~ /^#/ } @_;
   $class->_load_components(@comp);
 }
 
