@@ -9,7 +9,7 @@ sub delete {
 
   my $ret = $self->next::method(@rest);
 
-  my %rels = %{ $self->_relationships };
+  my %rels = map { $_ => $self->relationship_info($_) } $self->relationships;
   my @cascade = grep { $rels{$_}{attrs}{cascade_delete} } keys %rels;
   foreach my $rel (@cascade) {
     $self->search_related($rel)->delete;
@@ -24,7 +24,7 @@ sub update {
 
   my $ret = $self->next::method(@rest);
 
-  my %rels = %{ $self->_relationships };
+  my %rels = map { $_ => $self->relationship_info($_) } $self->relationships;
   my @cascade = grep { $rels{$_}{attrs}{cascade_update} } keys %rels;
   foreach my $rel (@cascade) {
     $_->update for $self->$rel;
